from django.urls import path

from . import views

urlpatterns = [
    path(
        'django_rrweb/record/events/',
        views.record_events,
        name='django-rrweb-record-events',
    ),
    path(
        'django_rrweb/record/script/',
        views.record_script,
        name='django-rrweb-record-script',
    ),
]



import datetime as dt
import json

from django.http import HttpResponse
from django.shortcuts import render
from django.utils import timezone as tz
from django.views.decorators.csrf import csrf_exempt

from .models import Event, Session


@csrf_exempt
def record_events(request):
    """Record events"""
    session = _get_session(request)
    obj = json.load(request)
    events = [
        Event(
            kind=event['type'],
            data=json.dumps(event['data']),
            timestamp=event['timestamp'],
            session=session,
        )
        for event in obj['rrwebEvents']
    ]
    Event.objects.bulk_create(events)
    return HttpResponse('OK', content_type='text/plain')


def _get_session(request):
    session_id = request.session.get('django_rrweb_session_id')
    if session_id is None:
        session = _new_session(request)
    else:
        try:
            session = Session.objects.get(id=session_id)
        except Session.DoesNotExist:
            session = _new_session(request)
    timestamp = session.max_timestamp()
    if not timestamp:
        return session
    event_time = tz.make_aware(dt.datetime.fromtimestamp(timestamp / 1000))
    if tz.now() - event_time > dt.timedelta(minutes=10):
        session = _new_session(request)
    return session


def _new_session(request):
    session = Session()
    session.save()
    session_id = session.id
    request.session['django_rrweb_session_id'] = session_id
    return session


def record_script(request):
    """Record script"""
    _get_session(request)
    return render(
        request,
        'django-rrweb/record-script.js',
        content_type='application/javascript',
    )



from django.contrib import admin
from django.db.models import Count, Max, Min, Sum
from django.db.models.functions import Length
from django.shortcuts import get_object_or_404, render
from django.urls import path, reverse
from django.utils.html import format_html

from .models import Event, Session


@admin.register(Event)
class EventAdmin(admin.ModelAdmin):
    list_display = ['session', 'timestamp']
    list_select_related = ['session']


@admin.register(Session)
class SessionAdmin(admin.ModelAdmin):
    list_display = [
        'create_time',
        'duration',
        'event_count',
        'event_data_size',
        'link_view',
    ]

    fields = [
        'create_time',
        'duration',
        'event_count',
        'event_data_size',
        'link_view',
    ]

    readonly_fields = fields

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        queryset = (
            queryset.annotate(event_count_num=Count('events__id'))
            .annotate(
                duration_num=(
                    Max('events__timestamp') - Min('events__timestamp')
                )
            )
            .annotate(event_data_size_num=Sum(Length('events__data')))
        )
        return queryset

    @admin.display(
        description='Duration',
        ordering='duration_num',
    )
    def duration(self, obj):
        return obj.duration()

    @admin.display(
        description='Event Count',
        ordering='event_count_num',
    )
    def event_count(self, obj):
        return obj.event_count()

    @admin.display(
        description='Event Data Size',
        ordering='event_data_size_num',
    )
    def event_data_size(self, obj):
        return obj.event_data_size()

    @admin.display(
        description='View',
        ordering='create_time',
    )
    def link_view(self, obj):
        url = reverse(
            'admin:django-rrweb-session-replay',
            kwargs={'session_id': obj.id},
        )
        return format_html('<a href="{url}">view</a>', url=url)

    def replay_view(self, request, session_id):
        session = get_object_or_404(Session, id=session_id)
        events = session.events.order_by('id')
        context = dict(
            self.admin_site.each_context(request),
            events=events,
            session=session,
        )
        return render(request, 'django-rrweb/session-replay.html', context)

    def get_urls(self):
        urls = super().get_urls()
        replay_path = path(
            '<session_id>/replay/',
            self.replay_view,
            name='django-rrweb-session-replay',
        )
        urls.insert(0, replay_path)
        return urls



from django.db import models
from django.db.models import Sum
from django.db.models.functions import Length


class Session(models.Model):
    create_time = models.DateTimeField(auto_now_add=True)

    def min_timestamp(self):
        timestamp = (
            self.events.order_by('timestamp')
            .values_list('timestamp', flat=True)
            .first()
        )
        return timestamp or 0

    def max_timestamp(self):
        timestamp = (
            self.events.order_by('-timestamp')
            .values_list('timestamp', flat=True)
            .first()
        )
        return timestamp or 0

    def duration(self):
        return self.max_timestamp() - self.min_timestamp()

    def event_count(self):
        return self.events.count()

    def event_data_size(self):
        return self.events.aggregate(size=Sum(Length('data')))['size']

    def __str__(self):
        return f'#{self.id} @ {self.create_time}'


class Event(models.Model):
    session = models.ForeignKey(
        Session,
        on_delete=models.CASCADE,
        related_name='events',
    )
    kind = models.SmallIntegerField()
    data = models.TextField(blank=True)
    timestamp = models.BigIntegerField()

    def __str__(self):
        return f'#{self.id} @ {self.timestamp}'



"""Package Setup for Django Rrweb
"""

import pathlib
import re

import setuptools
from setuptools.command.test import test as TestCommand


class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import tox

        errno = tox.cmdline(self.test_args)
        exit(errno)


init = (pathlib.Path('django_rrweb') / '__init__.py').read_text()
match = re.search(r"^__version__ = '(.+)'$", init, re.MULTILINE)
version = match.group(1)

with open('README.rst') as reader:
    readme = reader.read()

setuptools.setup(
    name='django-rrweb',
    version=version,
    description='Record and replay client web sessions.',
    long_description=readme,
    author='Grant Jenks',
    author_email='contact@grantjenks.com',
    url='https://grantjenks.com/docs/django-rrweb/',
    license='Apache 2.0',
    packages=['django_rrweb'],
    include_package_data=True,
    tests_require=['tox'],
    cmdclass={'test': Tox},
    install_requires=['Django'],
    project_urls={
        'Documentation': 'https://grantjenks.com/docs/django-rrweb/',
        'Source': 'https://github.com/grantjenks/django-rrweb',
        'Tracker': 'https://github.com/grantjenks/django-rrweb/issues',
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
)
